--[[
    Stable by LordFarlander
    Version 2.1
    $Revision: 77538 $

    Based on Kennel.
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local L = LibStub( "AceLocale-3.0" ):GetLocale( "Stable" );

if( not LibStub( "LibPetAndMountDatabase-1.1", true ) ) then
    error( "Stable requires LibPetAndMountDatabase-1.1" );
    return;
end--if

local LibLordFarlander = LibStub( "LibLordFarlander-2.0", true );
if( not LibLordFarlander ) then
    error( L["%s requires %s"]:format( L["Coconuts"], "LibLordFarlander-2.0" ) );
    return;
end--if

Stable = LibStub( "AceAddon-3.0" ):NewAddon( L["Stable"], "AceEvent-3.0" );
Stable.version = tonumber( string.match( "$Rev: 77538 $", "(%d+)" ) );

Stable.bankopen = false;
Stable.swapped = false;
Stable.sets = {};
Stable.ui = { currentSelection = 0, };

function Stable:OnInitialize()
    local defaults = {
            profile = {
                --List of items to never swap
                NoSwap = {},
                --List of sets to swap
                DoSwap = {
                    ["MiniPets"] = true,
                        ["MiniPets.Normal"] = true,
                        ["MiniPets.Reagented"] = true,
                        ["MiniPets.Equipment"] = true,
                    ["Mounts"] = true,
                        ["Mounts.Ground"] = true,
                            ["Mounts.Ground.Slow"] = true,
                            ["Mounts.Ground.Fast"] = true,
                            ["Mounts.Ground.VeryFast"] = true,
                        ["Mounts.Flying"] = true,
                            ["Mounts.Flying.Fast"] = true,
                            ["Mounts.Flying.VeryFast"] = true,
                            ["Mounts.Flying.ExtremelyFast"] = true,
                },
            },
        };
    self.db = LibStub( "AceDB-3.0" ):New( "StableDB", defaults );
end--Stable:OnInitialize()

function Stable:OnEnable()
    local PetAndMountDatabase = LibStub( "LibPetAndMountDatabase-1.1" );
    local AceConfigDialog = LibStub( "AceConfigDialog-3.0" );
    local Sets = { PetAndMountDatabase:GetSubsets( "MiniPets" ) };
    local MountSubsets = PetAndMountDatabase:GetSubsets( "Mounts" );
    local AllSets = {};
    local neverSwapOptions = {
            type = "group",
            name = L["Never Swap"],
            desc = L["Items that should never be swapped."],
            handler = self;
            args = {
                Items = {
                    type = "select",
                    order = 1,
                    name = L["Items"],
                    desc = L["Items"],
                    width = "full",
                    values = function( info )
                                 local numContinents = #GetMapContinents();
                                 local items = {};

                                 for itemID, _ in pairs( info.handler.db.profile.NoSwap ) do
                                     items[itemID] = GetItemInfo( itemID ) or itemID;
                                 end--for
                                 if( #items ) then
                                     if( ( info.handler.ui.currentSelection == 0 ) or ( not info.handler.db.profile.NoSwap[info.handler.ui.currentSelection] ) ) then
                                         info.handler.ui.currentSelection = 1;
                                     end--if
                                 else
                                     info.handler.ui.currentSelection = 0;
                                 end--if
                                 return items;
                             end,
                    get = function( info )
                              return info.handler.ui.currentSelection;
                          end,
                    set = function( info, val )
                              info.handler.ui.currentSelection = val;
                          end,
                },
                Item = {
                    type = "input",
                    order = 2,
                    name = L["Item"],
                    desc = L["Item"],
                    width = "full",
                    get = function( info )
                              return info.handler.db.profile.NoSwap[info.handler.ui.currentSelection] or "";
                          end,
                    set = function( info, val )
                              local newItemID = LibLordFarlander.GetItemIDFromLink( LibLordFarlander.GetItemLink( val ) );
                              
                              if( newItemID ) then
                                  info.handler.db.profile.NoSwap[newItemID] = true;
                              end--if
                          end,
                },
                Delete = {
                    type = "execute",
                    order = 3,
                    name = L["Delete"],
                    desc = L["Delete the currently selected item from the list."],
                    func = function( info )
                               if( info.handler.ui.currentSelection > 0 ) then
                                   table.remove( info.handler.db.profile.NoSwap, info.handler.ui.currentSelection );
                               end--if
                           end,
                },                        
            },
        };
    local options = { handler = self, type = "group", args = {},
            name = L["Stable"],
            desc = L["Swap a random minipet and/or mount every time you visit the bank"],
            icon = "Interface\\Icons\\INV_Box_PetCarrier_01",
        };
    local optionsSet = {};

    self:RegisterEvent( "BANKFRAME_OPENED" );
    self:RegisterEvent( "BANKFRAME_CLOSED",
        function()
            if( ( not Stable.bankopen ) and ( not Stable.swapped ) ) then
                Stable:BANKFRAME_OPENED();
            end--if
            if( Stable.bankopen and Stable.swapped and FuBar_CorkFu ) then
                FuBar_CorkFu:GetModule( "Minipet" ):ActivatePet();
            end--if
            Stable.bankopen = false;
        end );

    for _, set in pairs( MountSubsets ) do
        table.insert( Sets, PetAndMountDatabase:GetSubsets( set ) );
    end--for
    -- We want to make sure not to swap Quests and Children and remove the base set
    for _, set in pairs( Sets ) do
        self.CheckSets( set, AllSets );
    end--for
    for _, set in pairs( AllSets ) do
        local setName = set;
        local setParts = { strsplit( ".", set ) };
        local commandString = L["swap"];
        local shortString = "";
        local longString = "";
        local stableSets = self.sets;
        local commandSet = "";
        local setPartsCount = #setParts;

        if( setPartsCount > 1 ) then
            setName = setParts[1];
        end--if
        if( stableSets[setName] == nil ) then
            stableSets[setName] = {};
        end--if
        stableSets[setName][set] = { player = { bag = {}, slot = {}, num = 0 }, bank = { bag = {}, slot = {}, num = 0 } };
        if( setPartsCount >= 3 ) then
            stableSets[setName][set].subSet = table.concat( setParts, ".", 1, setPartsCount - 1 );
        end--if
        -- Build commands
        for _, setSubname in pairs( setParts ) do
            local shortName = setSubname:lower();
            local longName = setSubname:lower();
            local commandName = setSubname;

            if( L[setSubname .. "_short"] ) then
                shortName = L[setSubname .. "_short"];
            end--if
            if( L[setSubname .. "_long"] ) then
                longName = L[setSubname .. "_long"];
            end--if
            if( L[setSubname .. "_command"] ) then
                commandName = L[setSubname .. "_command"];
            end--if
            commandString = commandString .. commandName;
            if( shortString:len() > 0 ) then
                shortString = " " .. shortString;
            end--if
            shortString = shortName .. shortString;
            if( longString:len() > 0 ) then
                longString = " " .. longString;
            end--if
            longString = longName .. longString;
            if( commandSet:len() > 0 ) then
                commandSet = commandSet .. ".";
            end--if
            commandSet = commandSet .. setSubname;
            if( self.db.profile.DoSwap[commandSet] == nil ) then
                self.db.profile.DoSwap[commandSet] = true;
            end--if
            if( options.args[commandString] == nil ) then
                table.insert( optionsSet, commandString );
                options.args[commandString] = {
                    type = "toggle",
                    name = L["Swap %s"]:format( shortString ),
                    desc = L["Toggles the swapping of %s."]:format( longString ),
                    get = loadstring( ("return Stable.db.profile.DoSwap[%q];"):format( commandSet ) ),
                    set = loadstring( ("Stable.db.profile.DoSwap[%q] = not Stable.db.profile.DoSwap[%q];"):format( commandSet, commandSet ) ),
                    };
            end--if
        end--for
    end--for
    table.sort( optionsSet );
    for index, option in pairs( optionsSet ) do
        options.args[option].order = index;
    end--for
	LibStub( "AceConfig-3.0" ):RegisterOptionsTable( L["Stable"], options, { L["stable"], L["stbl"] } );
    LibStub( "AceConfigRegistry-3.0" ):RegisterOptionsTable( L["Stable Never Swap"], neverSwapOptions );
	AceConfigDialog:AddToBlizOptions( L["Stable"] );
    AceConfigDialog:AddToBlizOptions( L["Stable Never Swap"], L["Never Swap"], L["Stable"] );

    --Upgrade check
    --Convert old save values
    if( self.db.profile.DoSwap["GroundMounts"] ~= nil ) then
        self.db.profile.DoSwap["Mounts.Ground"] = self.db.profile.DoSwap["GroundMounts"];
        self.db.profile.DoSwap["GroundMounts"] = nil;
    end--if
    if( self.db.profile.DoSwap["FlyingMounts"] ~= nil ) then
        self.db.profile.DoSwap["Mounts.Flying"] = self.db.profile.DoSwap["FlyingMounts"];
        self.db.profile.DoSwap["FlyingMounts"] = nil;
    end--if
    if( self.db.profile.DoSwap["MiniPets.Holiday"] ~= nil ) then
        self.db.profile.DoSwap["MiniPets.Reagented"] = self.db.profile.DoSwap["MiniPets.Holiday"];
        self.db.profile.DoSwap["MiniPets.Holiday"] = nil;
    end--if
    if( self.db.profile.LastVersion ~= self.version ) then
        self.db.profile.LastVersion = self.version;
    end--if
end--Stable:OnEnable()

function Stable.CheckSets( sets, dst )
    for _, name in pairs( sets ) do
        if( ( not name:find( ".Quest" ) ) and ( not name:find( ".Children" ) ) ) then
            local start, ends = name:find( "PetAndMountDatabase." );
            
            if( ends ) then
                name = name:sub( ends + 1 );
            end--if
            table.insert( dst, name );
        end--if
    end--for
end--Stable.CheckSets( sets, dst )

function Stable:BANKFRAME_OPENED()
    local doSwap = self.db.profile.DoSwap;
    local stableSets = self.sets;
    local PetAndMountDatabase = LibStub( "LibPetAndMountDatabase-1.1" );
    local NoSwap = self.db.profile.NoSwap;

    self.bankopen = true;
    self.swapped = false;

    for _, set in pairs( stableSets ) do
       for _, subset in pairs( set ) do
           subset.player = { bag = {}, slot = {}, num = 0 };
           subset.bank = { bag = {}, slot = {}, num = 0 };
       end--for
    end--for
    for bag = BANK_CONTAINER, NUM_BAG_SLOTS + NUM_BANKBAGSLOTS do
        for slot = 1, GetContainerNumSlots( bag ) do
            local itemLink = GetContainerItemLink( bag, slot );
            local bagSet = ( bag <= NUM_BAG_SLOTS and bag >= 0 ) and "player" or "bank";

            if( itemLink ) then
                local itemID = LibLordFarlander.GetItemIDFromLink( itemLink );

                for setName, set in pairs( stableSets ) do
                    if( doSwap[setName] ) then
                        for subsetName, subset in pairs( set ) do
                            local doSwapSet = doSwap[subsetName];

                            if( subset.subSet ) then
                                doSwapSet = doSwapSet and doSwap[subset.subSet];
                            end--if
--                          ( swappingMiniPets and bit.band( GetItemFamily( itemID ), 4096 ) ~= 0 ) )
                            if( doSwapSet and ( not NoSwap[itemID] ) and PetAndMountDatabase:ItemInSet( itemID, subsetName ) ) then
                                local petSet = subset[bagSet];
                                local n = petSet.num + 1;
        
                                petSet.num = n;
                                petSet.bag[n] = bag;
                                petSet.slot[n] = slot;
                            end--if
                        end--for
                    end--if
                end--for
            end--if
        end--for
    end--for
    for setName, set in pairs( stableSets ) do
        if( doSwap[setName] ) then
            for subsetName, subset in pairs( set ) do
                local doSwapSet = doSwap[subsetName];

                if( set.subSet ) then
                    doSwapSet = doSwapSet and doSwap[set.subSet];
                end--if                
                if( doSwapSet ) then
                    self.swapped = self:DoBagSwap( subset ) or self.swapped;
                end--if
            end--for
        end--if
    end--for
end--Stable:BANKFRAME_OPENED()

------------------------------
--     Helper Functions     --
------------------------------
function Stable:DoBagSwap( set )
    if set.player.num > set.bank.num then
        set.player, set.bank = set.bank, set.player;
    end--if

    local p, b = set.player, set.bank;

    if( p.num == 0 ) then
        return false;
    end--if
    for i = 1, p.num do
        local r = math.random( b.num + 1 - i );

        PickupContainerItem( p.bag[i], p.slot[i] );
        PickupContainerItem( b.bag[r], b.slot[r] );
        table.remove( b.bag, r );
        table.remove( b.slot, r );
    end--for
    return ( p.num > 0 ) and ( b.num > 0 );
end--Stable:DoSwaps( set )

function Stable:DoInventorySwap( set )
    local tabardSet = type.normal;
    local equipSet = set.playerEquipment;

    if( set.bankEquipment.num ) then
        equipSet = set.bankEquipment;
    end--if
    
    if( equipSet.num == 0 ) then
        return false;
    end--if

    local r = math.random( equipSet.num );

    PickupContainerItem( equipSet.bag[r], equipSet.slot[r] );
    AutoEquipCursorItem();
    return( true );
end--Stable:DoInventorySwap( type )

